"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.memo = void 0;
function memo(opts) {
    const get_key = opts.key ?? default_key;
    const storage = Object.create(null);
    function memoized(...args) {
        const key = get_key(args);
        if (storage[key]) {
            storage[key].hits++;
            return storage[key].result;
        }
        const result = opts.func(...args);
        store_record(key, args, result);
        return result;
    }
    function evict(key) {
        if (storage[key]) {
            const evicted = storage[key];
            delete storage[key];
            if (evicted.timer) {
                clearTimeout(evicted.timer);
                evicted.timer = null;
            }
            if (opts.on_evict) {
                opts.on_evict(evicted);
            }
        }
    }
    function store_record(key, params, result) {
        const record = {
            key,
            created: Date.now(),
            expires: null,
            result,
            hits: 0,
            timer: null,
        };
        const ttl = typeof opts.ttl === 'function'
            ? opts.ttl(params, result)
            : opts.ttl;
        record.expires = record.created + ttl;
        record.timer = setTimeout(() => evict(record.key), ttl);
    }
    memoized.invalidate = function invalidate(...params) {
        const key = get_key(params);
        evict(key);
    };
    return memoized;
}
exports.memo = memo;
function default_key(...params) {
    return JSON.stringify(params);
}
//# sourceMappingURL=full.js.map